﻿$(function ()
{
    diagramHub = $.connection.diagramHub;
    diagramHub.client.nodeCreated = function (model)
    {
        var node = diagram.factory.createShapeNode(
            model.x, model.y, model.width, model.height);
        node.id = model.id;
    };
    diagramHub.client.nodeModified = function (model)
    {
        var node = findNode(model.id);
        node.setBounds(
            new MindFusion.Drawing.Rect(
                model.x, model.y, model.width, model.height),
            true);
    };
    diagramHub.client.nodeTextEdited = function (model)
    {
        var node = findNode(model.id);
        node.text = model.text;
    };
    diagramHub.client.nodeDeleted = function (model)
    {
        var node = findNode(model.id);
        diagram.removeItem(node);
    };
    diagramHub.client.linkCreated = function (model)
    {
        var link = diagram.factory.createDiagramLink(
            findNode(model.originId), findNode(model.destinationId));
        link.id = model.id;
    };
    diagramHub.client.linkModified = function (model)
    {
        var link = findLink(model.id);
        link.origin = findNode(model.originId);
        link.destination = findNode(model.destinationId);
    };
    diagramHub.client.linkTextEdited = function (model)
    {
        var link = findLink(model.id);
        link.text = model.text;
    };
    diagramHub.client.linkDeleted = function (model)
    {
        var link = findLink(model.id);
        diagram.removeItem(link);
    };
    diagramHub.client.load = function (url)
    {
        fetch(url)
	    .then(response => response.json())
	    .then((data) => {
	    	diagram.fromJson(JSON.stringify(data))
	    })
	    .catch(err => { throw err });
    };

    $.connection.hub.start();
});

function onDiagramLoaded(s, e)
{
    diagram = s.diagram;
}

function onNodeCreated(s, e)
{
    var hubId = $.connection.hub.id;
    e.node.id = hubId + new Date().valueOf();
    
    var r = e.node.bounds;
    var model =
    {
        id: e.node.id,
        x: r.x,
        y: r.y,
        width: r.width,
        height: r.height
    };
    
    diagramHub.server.nodeCreated(model);
}

function onNodeModified(s, e)
{
    var r = e.node.bounds;
    var model =
    {
        id: e.node.id,
        x: r.x,
        y: r.y,
        width: r.width,
        height: r.height
    };
    diagramHub.server.nodeModified(model);
}

function onNodeTextEdited(s, e)
{
    var model =
    {
        id: e.node.id,
        text: e.newText
    };
    diagramHub.server.nodeTextEdited(model);
}

function onNodeDeleted(s, e)
{
    var model =
    {
        id: e.node.id
    };
    diagramHub.server.nodeDeleted(model);
}

function onLinkCreated(s, e)
{
    var hubId = $.connection.hub.id;
    e.link.id = hubId + new Date().valueOf();
   
    var model =
    {
        id: e.link.id,
        originId: e.link.origin.id,
        destinationId: e.link.destination.id,
    };
    
    diagramHub.server.linkCreated(model);
}

function onLinkModified(s, e)
{
    var hubId = $.connection.hub.id;
    var model =
    {
        id: e.link.id,
        originId: e.link.origin.id,
        destinationId: e.link.destination.id,
    };
    diagramHub.server.linkModified(model);
}

function onLinkTextEdited(s, e)
{
    var model =
    {
        id: e.link.id,
        text: e.newText
    };
    diagramHub.server.linkTextEdited(model);
}

function onLinkDeleted(s, e)
{
    var model =
    {
        id: e.link.id
    };
    diagramHub.server.linkDeleted(model);
}

function findNode(id)
{
    for (var i = 0; i < diagram.nodes.length; i++)
    {
        var node = diagram.nodes[i];
        if (id == node.id)
            return node;
    }
    return null;
}

function findLink(id)
{
    for (var i = 0; i < diagram.links.length; i++)
    {
        var link = diagram.links[i];
        if (id == link.id)
            return link;
    }
    return null;
}

function onLoad()
{
    var url = "./Diagram.json";
    diagramHub.server.load(url);
}

